package com.yunzhi.niucai.alipay;


import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.request.AlipayOpenAuthTokenAppRequest;
import com.alipay.api.request.AlipayTradeAppPayRequest;
import com.alipay.api.response.AlipayOpenAuthTokenAppResponse;
import com.alipay.api.response.AlipayTradeAppPayResponse;
import com.yunzhi.niucai.alipay.vo.PayParam;
import lombok.extern.slf4j.Slf4j;

import java.util.HashMap;
import java.util.Map;

@Slf4j
public class PayService {
    private ITokenResolve iTokenResolve;
    private AlipayClient alipayClient;
    private Config config;
    private Integer shopId;

    public PayService(Integer shopId, AlipayClient alipayClient, ITokenResolve iTokenResolve, Config config) {
        this.shopId = shopId;
        this.iTokenResolve = iTokenResolve;
        this.alipayClient = alipayClient;
        this.config = config;
    }

    /**
     * 支付
     * @param payParam
     * @return
     * @throws AlipayApiException
     */
    public AlipayTradeAppPayResponse Pay(PayParam payParam) throws AlipayApiException {
        AlipayTradeAppPayRequest aliPayRequest = new AlipayTradeAppPayRequest();

        String token = null;
        if (this.iTokenResolve.isTokenExpired(this.shopId)) {
            AlipayOpenAuthTokenAppResponse refreshResponse = refreshToken();
            if (null != refreshResponse) {
                token = refreshResponse.getAppAuthToken();
            }
        } else {
            token = this.iTokenResolve.getAuthToken(this.shopId);
        }

        aliPayRequest.putOtherTextParam("app_auth_token", token);
        aliPayRequest.setNotifyUrl(this.config.getNotifyURL());

        if (null == payParam.getTimeoutExpress()) {
            payParam.setTimeoutExpress("30m");
        }

        aliPayRequest.setBizContent(JSONObject.toJSONString(payParam));
        try {
            AlipayTradeAppPayResponse response = alipayClient.sdkExecute(aliPayRequest);
            if (response.isSuccess()) {
                return response;
            } else {
                log.error(String.format("支付宝下单失败, 商户ID %s : 错误码[%s], 错误说明[%s]", this.shopId, response.getSubCode(), response.getSubMsg()));
                return null;
            }
        } catch (AlipayApiException e) {
            log.error(String.format("使用支付宝下单失败: %s", e.getErrMsg()));
            e.printStackTrace();
            throw e;
        }
    }

    /**
     * 使用 code 交换获取 token
     * @param code
     * @throws AlipayApiException
     */
    public void persistToken(String code) throws AlipayApiException {
        AlipayOpenAuthTokenAppRequest request = new AlipayOpenAuthTokenAppRequest();
        Map<String, String> param = new HashMap<String, String>() {{
            put("grant_type", "authorization_code");
            put("code", code);
        }};

        request.setBizContent(JSONObject.toJSONString(param));
        AlipayOpenAuthTokenAppResponse response = alipayClient.execute(request);
        if(response.isSuccess()){
            this.iTokenResolve.persistAuthToken(response.getAuthAppId(),
                    response.getAuthAppId(),
                    response.getAppAuthToken(),
                    response.getAppRefreshToken(),
                    response.getExpiresIn());
        } else {
            log.error(String.format("获取支付宝 APP_AUTH_TOKEN 失败, 商户ID %s : 错误码[%s], 错误说明[%s]", this.shopId, response.getSubCode(), response.getSubMsg()));
        }
    }

    private AlipayOpenAuthTokenAppResponse refreshToken() throws AlipayApiException {
        AlipayOpenAuthTokenAppRequest request = new AlipayOpenAuthTokenAppRequest();
        String refreshToken = this.iTokenResolve.getRefreshToken(this.shopId);
        Map<String, String> param = new HashMap<String, String>() {{
            put("grant_type", "refresh_token");
            put("refresh_token", refreshToken);
        }};

        request.setBizContent(JSONObject.toJSONString(param));
        AlipayOpenAuthTokenAppResponse response = alipayClient.execute(request);
        if(response.isSuccess()){
            this.iTokenResolve.refreshAuthToken(this.shopId,
                    response.getAppAuthToken(),
                    response.getAppRefreshToken(),
                    response.getExpiresIn());
            return response;
        } else {
            log.error(String.format("刷新支付宝 APP_AUTH_TOKEN 失败, 商户ID %s : 错误码[%s], 错误说明[%s]", this.shopId, response.getSubCode(), response.getSubMsg()));
            return null;
        }
    }
}
