package com.yunzhi.niucai.controller;

import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.request.AlipayOpenAuthTokenAppRequest;
import com.alipay.api.response.AlipayOpenAuthTokenAppResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yunzhi.niucai.alipay.AlipayFactory;
import com.yunzhi.niucai.alipay.PayService;
import com.yunzhi.niucai.common.BaseController;
import com.yunzhi.niucai.common.ResponseBean;
import com.yunzhi.niucai.entity.TaAlipay;
import com.yunzhi.niucai.entity.TaOrder;
import com.yunzhi.niucai.entity.TaShopApp;
import com.yunzhi.niucai.service.ITaShopAppService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.yunzhi.niucai.service.ITaAlipayService;

import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;

/**
 * <p>
    *  前端控制器
    * </p>
 *
 * @author yansen
 * @since 2020-09-27
 */

@Api(tags = "")
@RestController
@RequestMapping("/aliPay")
public class TaAlipayController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaAlipayController.class);


    @Autowired
    public ITaShopAppService iTaShopAppService;

    @Autowired
    private AlipayFactory alipayFactory;

    @Autowired
    private ITaAlipayService aliPayService;

    /**
     * 手机app获取唤醒支付宝的参数
     * @param request
     * @param taOrder
     * @return
     * @throws Exception
     */
    @PostMapping("/api/app/alipay/order")
    @ApiOperation(value = "创建订单", notes = "创建订单", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean create(HttpServletRequest request,@ApiParam("下单记录") @RequestBody TaOrder taOrder) throws Exception {
        boolean res = aliPayService.orderPaying(taOrder);
        if (res) {
            return ResponseBean.success("success");
        } else {
            return ResponseBean.error("error", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    @GetMapping("/api/app/alipay/result")
    @ApiOperation(value = "查询支付状态", notes = "查询支付状态", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getPayResult(@ApiParam("订单号") @RequestParam String orderId) {
        return ResponseBean.success("");
    }

    /**
     * 支付成功回调
     * @param request
     */
    @GetMapping("/alipay/pay/callback")
    public void callback(HttpServletRequest request){
        String tradeNo = request.getParameter("trade_no");
        String appId = request.getParameter("app_id");
        String outTradeNo = request.getParameter("out_trade_no");
        String tradeStatus = request.getParameter("trade_status");
        aliPayService.payCallback(tradeNo,outTradeNo,tradeStatus);
    }

    /**
     * 应用授权成功回调
     * @param appId
     * @param appAuthCode
     * @throws AlipayApiException
     */
    @GetMapping("/alipay/auth/callback")
    public void aliPayAuthNotice(@RequestParam("app_id") String appId,@RequestParam("app_auth_code") String appAuthCode) throws AlipayApiException {
        // 虚拟一个服务, 主要是为了调用其中方法
        PayService payService = alipayFactory.getService(-1);
        payService.persistToken(appAuthCode);
    }
}
