package com.yunzhi.niucai.service.impl;

import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.request.AlipayTradeAppPayRequest;
import com.alipay.api.response.AlipayTradeAppPayResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yunzhi.niucai.alipay.AlipayFactory;
import com.yunzhi.niucai.alipay.PayService;
import com.yunzhi.niucai.alipay.vo.PayParam;
import com.yunzhi.niucai.common.CommConstants;
import com.yunzhi.niucai.common.ResponseBean;
import com.yunzhi.niucai.entity.TaAlipay;
import com.yunzhi.niucai.entity.TaOrder;
import com.yunzhi.niucai.entity.TaShopApp;
import com.yunzhi.niucai.mapper.TaAlipayMapper;
import com.yunzhi.niucai.service.ITaAlipayService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yunzhi.niucai.service.ITaOrderService;
import com.yunzhi.niucai.service.ITaShopAppService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.math.BigInteger;
import java.security.SecureRandom;
import java.time.LocalDateTime;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author yansen
 * @since 2020-09-27
 */
@Service
public class TaAlipayServiceImpl extends ServiceImpl<TaAlipayMapper, TaAlipay> implements ITaAlipayService {

    @Autowired
    public ITaShopAppService iTaShopAppService;

    @Autowired
    private ITaOrderService taOrderService;

    @Autowired
    private AlipayFactory alipayFactory;

    @Autowired
    private TaAlipayMapper taAlipayMapper;

    /**
     * 订单支付中
     * @param taOrder
     * @return
     * @throws AlipayApiException
     */
    @Override
    public boolean orderPaying(TaOrder taOrder) throws AlipayApiException {
        PayService payService = alipayFactory.getService(taOrder.getShopId());

        String outTradeNo = taOrder.getOrderId() + "-" + randomString(3);
        String subject = "支付宝支付";
        String totalAmount = String.format("%.2f", (float) taOrder.getCharge() / 100);
        PayParam payParam = new PayParam()
                .setOutTradeNo(outTradeNo)
                .setSubject(subject)
                .setTotalAmount(totalAmount);

        AlipayTradeAppPayResponse response = payService.Pay(payParam);
        if (null == response) {
            return false;
        }

        taOrder.setOutTradeNo(outTradeNo);
        taOrder.setStatus(CommConstants.SRTATUS_PAYING);
        return taOrderService.updateById(taOrder);
    }

    /**
     * 订单回调
     * @param tradeNo
     * @param outTradeNo
     * @param tradeStatus
     */
    @Override
    public void payCallback(String tradeNo, String outTradeNo, String tradeStatus) {
        TaOrder taOrder = taOrderService.getOne(new QueryWrapper<TaOrder>().eq("out_trade_no", outTradeNo));
        taOrder.setUpdateDate(LocalDateTime.now());
        // 支付失败
        if (!tradeStatus.equals(CommConstants.TRADE_STATUS_SUCCESS)){
            taOrder.setStatus(CommConstants.SRTATUS_EFFECTIVE);
            taOrderService.updateById(taOrder);
            return;
        }
        //支付成功 更新为已支付状态
        taOrder.setStatus(CommConstants.SRTATUS_EFFECTIVE);
        taOrder.setTradeNo(tradeNo);
        taOrderService.updateById(taOrder);
    }

    /**
     * 生成随机数
     * @param length
     * @return
     */
    public String randomString(int length) {
        String str = new BigInteger(130, new SecureRandom()).toString(32);
        return str.substring(0, length);
    }
}
