package com.yunzhi.niucai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yunzhi.niucai.alipay.ITokenResolve;
import com.yunzhi.niucai.common.StringUtils;
import com.yunzhi.niucai.entity.TaShopApp;
import com.yunzhi.niucai.mapper.TaShopAppMapper;
import com.yunzhi.niucai.service.ITaShopAppService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

/**
 * <p>
 * 门店应用表  服务实现类
 * </p>
 *
 * @author yansen
 * @since 2020-09-23
 */
@Service
public class TaShopAppServiceImpl extends ServiceImpl<TaShopAppMapper, TaShopApp> implements ITaShopAppService, ITokenResolve {

    @Override
    public boolean isTokenExpired(Integer shopId) {
        TaShopApp taShopApp = getByShopId(shopId);
        String expires = taShopApp.getExpiresIn();
        if (StringUtils.isEmpty(expires)) {
            expires = "0";
        }

        long expiresIn = Long.parseLong(expires);
        return taShopApp.getUpdateTokenDate().plusSeconds(expiresIn).isBefore(LocalDateTime.now());
    }

    private TaShopApp getByShopId(Integer shopId) {
        QueryWrapper<TaShopApp> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("shop_id", shopId);
        
        return getOne(queryWrapper);
    }

    private TaShopApp getByAppId(String appId) {
        QueryWrapper<TaShopApp> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("appid", appId);

        return getOne(queryWrapper);
    }

    @Override
    public String getAuthToken(Integer shopId) {
        return getByShopId(shopId).getAppAuthToken();
    }

    @Override
    public String getRefreshToken(Integer shopId) {
        return getByShopId(shopId).getAppRefreshToken();
    }

    @Override
    public void persistAuthToken(String appId, String alipayId, String authToken, String refreshToken, String expiresIn) {
        TaShopApp taShopApp = getByAppId(appId);
        taShopApp.setAppAuthToken(authToken);
        taShopApp.setAppRefreshToken(refreshToken);
        taShopApp.setExpiresIn(expiresIn);
        taShopApp.setAlipayId(alipayId);
        taShopApp.setUpdateTokenDate(LocalDateTime.now());

        updateById(taShopApp);
    }

    @Override
    public void refreshAuthToken(Integer shopId, String authToken, String refreshToken, String expiresIn) {
        TaShopApp taShopApp = getByShopId(shopId);
        taShopApp.setAppAuthToken(authToken);
        taShopApp.setAppRefreshToken(refreshToken);
        taShopApp.setExpiresIn(expiresIn);
        taShopApp.setUpdateTokenDate(LocalDateTime.now());

        updateById(taShopApp);
    }
}
